@@ -20,131 +20,23 @@
  */
 
 #include <config.h>
+
 #include <string.h>
 #include <time.h>
 #include <sys/types.h>
 #include <regex.h>
-#include <gconf/gconf-client.h>
+
 #include <glib/gi18n.h>
+#include <libxml/uri.h>
+
 #include "gossip-utils.h"
+#include "gossip-conf.h"
+#include "gossip-paths.h"
+#include "gossip-debug.h"
 
-#define DEBUG_MSG(x)   
-/* #define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n");   */
+#define DEBUG_DOMAIN "Utils"
 
-static void regex_init                  (void);
-static void status_message_free_foreach (GossipStatusEntry *entry);
-
-
-GList *
-gossip_status_messages_get (void)
-{
-	GConfClient       *gconf_client;
-	GSList            *list, *l;
-	GList             *ret = NULL;
-	GossipStatusEntry *entry;
-	const gchar       *status;
-	
-	gconf_client = gconf_client_get_default ();
-	
-	list = gconf_client_get_list (gconf_client,
-				      "/apps/gossip/status/preset_messages",
-				      GCONF_VALUE_STRING,
-				      NULL);
-
-	g_object_unref (gconf_client);
-
-	/* This is really ugly, but we can't store a list of pairs and a dir
-	 * with entries wouldn't work since we have no guarantee on the order of
-	 * entries.
-	 */
-	
-	for (l = list; l; l = l->next) {
-		entry = g_new (GossipStatusEntry, 1);
-
-		status = l->data;
-
-		if (strncmp (status, "available/", 10) == 0) {
-			entry->string = g_strdup (&status[10]);
-			entry->state = GOSSIP_PRESENCE_STATE_AVAILABLE;
-		}
-		else if (strncmp (status, "busy/", 5) == 0) {
-			entry->string = g_strdup (&status[5]);
-			entry->state = GOSSIP_PRESENCE_STATE_BUSY;
-		}
-		else if (strncmp (status, "away/", 5) == 0) {
-			entry->string = g_strdup (&status[5]);
-			entry->state = GOSSIP_PRESENCE_STATE_AWAY;
-		} else {
-			continue;
-		}
-		
-		ret = g_list_append (ret, entry);
-	}
-	
-	g_slist_foreach (list, (GFunc) g_free, NULL);
-	g_slist_free (list);
-	
-	return ret;
-
-}
-
-void 
-gossip_status_messages_set (GList *list)
-{
-	GConfClient       *gconf_client;
-	GList             *l;
-	GossipStatusEntry *entry;
-	GSList            *slist = NULL;
-	const gchar       *state;
-	gchar             *str;
-	
-	for (l = list; l; l = l->next) {
-		entry = l->data;
-
-		switch (entry->state) {
-		case GOSSIP_PRESENCE_STATE_AVAILABLE:
-			state = "available";
-			break;
-		case GOSSIP_PRESENCE_STATE_BUSY:
-			state = "busy";
-			break;
-		case GOSSIP_PRESENCE_STATE_AWAY:
-			state = "away";
-			break;
-		default:
-			state = NULL;
-			g_assert_not_reached ();
-		}
-		
-		str = g_strdup_printf ("%s/%s", state, entry->string);
-		slist = g_slist_append (slist, str);
-	}
-
-	gconf_client = gconf_client_get_default ();
-	gconf_client_set_list (gconf_client,
-			       "/apps/gossip/status/preset_messages",
-			       GCONF_VALUE_STRING,
-			       slist,
-			       NULL);
-	g_object_unref (gconf_client);
-
-	g_slist_foreach (slist, (GFunc) g_free, NULL);
-	g_slist_free (slist);
-}
-
-static void
-status_message_free_foreach (GossipStatusEntry *entry)
-{
-	g_free (entry->string);
-	g_free (entry);
-}
-
-void
-gossip_status_messages_free (GList *list)
-{
-	g_list_foreach (list, (GFunc) status_message_free_foreach, NULL);
-	g_list_free (list);
-}
+static void regex_init (void);
 
 gchar *
 gossip_substring (const gchar *str, 
@@ -245,7 +137,9 @@ gossip_regex_match (GossipRegExType  type,
 	}
 
  	if (type != GOSSIP_REGEX_ALL) { 
-		DEBUG_MSG (("Utils: Found %d matches for regex type:%d", num_matches, type));
+		gossip_debug (DEBUG_DOMAIN, 
+			      "Found %d matches for regex type:%d", 
+			      num_matches, type);
 		return num_matches;
 	}
 
@@ -267,7 +161,9 @@ gossip_regex_match (GossipRegExType  type,
 		}
 	}
 
-	DEBUG_MSG (("Utils: Found %d matches for ALL regex types", num_matches));
+	gossip_debug (DEBUG_DOMAIN, 
+		      "Found %d matches for ALL regex types", 
+		      num_matches);
 
 	return num_matches;
 }
@@ -301,19 +197,24 @@ gboolean
 gossip_xml_validate (xmlDoc      *doc, 
 		     const gchar *dtd_filename)
 {
-	gchar        *path;
+	gchar        *path, *escaped;
 	xmlValidCtxt  cvp;
 	xmlDtd       *dtd;
 	gboolean      ret;
 		
-	path = g_build_filename (DTDDIR, dtd_filename, NULL);
+	path = gossip_paths_get_dtd_path (dtd_filename);
 
+	/* The list of valid chars is taken from libxml. */
+	escaped = xmlURIEscapeStr (path, ":@&=+$,/?;");
+
+	g_free (path);
+	
 	memset (&cvp, 0, sizeof (cvp));
-	dtd = xmlParseDTD (NULL, path);
+	dtd = xmlParseDTD (NULL, escaped);
 	ret = xmlValidateDtd (&cvp, doc, dtd);
 	
+	xmlFree (escaped);
 	xmlFreeDtd (dtd);
-	g_free (path);
 	
         return ret;
 }
@@ -432,27 +333,3 @@ gossip_base64_encode (const guchar *data,
 
 	return rv;
 }
-
-
-gchar *
-gossip_sha1_string (const guchar *data,
-		    gsize         len) {
-	gchar  *hash_string;
-	gchar  *p;
-	guchar  hash[20];
-	gint    i;
-	
-	if (data == NULL || len < 1) {
-		return g_strdup("");
-	}
-	
-	gcry_md_hash_buffer (GCRY_MD_SHA1, hash, data, (size_t) len);
-	hash_string = g_malloc (sizeof (gchar) * 41);
-	p = hash_string;
-
-	for (i = 0; i < 20; i++, p += 2) {
-		snprintf (p, 3, "%02x", hash[i]);
-	}
-
-	return hash_string;
-}
