@@ -30,13 +30,8 @@
 #include <gtk/gtkstock.h>
 #include <gtk/gtkscrolledwindow.h>
 
-#ifdef USE_GNOMEVFS_FOR_URL
-#include <libgnomevfs/gnome-vfs.h>
-#else
-#include <libgnome/gnome-url.h>
-#endif
-
 #include <libgossip/gossip-debug.h>
+#include <libgossip/gossip-conf.h>
 #include <libgossip/gossip-time.h>
 #include <libgossip/gossip-utils.h>
 #include <libgossip/gossip-session.h>
@@ -47,8 +42,7 @@
 #include "gossip-preferences.h"
 #include "gossip-theme-manager.h"
 
-#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj),	\
-		       GOSSIP_TYPE_CHAT_VIEW, GossipChatViewPriv))
+#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_CHAT_VIEW, GossipChatViewPriv))
 
 #define DEBUG_DOMAIN "ChatView"
 
@@ -74,6 +68,10 @@ struct _GossipChatViewPriv {
 	BlockType      last_block_type;
 
 	gboolean       allow_scrolling;
+	gboolean       is_group_chat;
+	
+	GtkTextMark   *find_mark;
+	gboolean       find_wrapped;
 
 	/* This is for the group chat so we know if the "other" last contact
 	 * changed, so we know whether to insert a header or not.
@@ -87,97 +85,97 @@ struct _GossipChatViewPriv {
 typedef struct {
 	GossipSmiley  smiley;
 	const gchar  *pattern;
-	gint          index;
 } GossipSmileyPattern;
 
-static GossipSmileyPattern smileys[] = {
-	{ GOSSIP_SMILEY_NORMAL,       ":)",  0 },
-	{ GOSSIP_SMILEY_WINK,         ";)",  0 },
-	{ GOSSIP_SMILEY_WINK,         ";-)", 0 },
-	{ GOSSIP_SMILEY_BIGEYE,       "=)",  0 },
-	{ GOSSIP_SMILEY_NOSE,         ":-)", 0 },
-	{ GOSSIP_SMILEY_CRY,          ":'(", 0 },
-	{ GOSSIP_SMILEY_SAD,          ":(",  0 },
-	{ GOSSIP_SMILEY_SAD,          ":-(", 0 },
-	{ GOSSIP_SMILEY_SCEPTICAL,    ":/",  0 },
-	{ GOSSIP_SMILEY_SCEPTICAL,    ":\\", 0 },
-	{ GOSSIP_SMILEY_BIGSMILE,     ":D",  0 },
-	{ GOSSIP_SMILEY_BIGSMILE,     ":-D", 0 },
-	{ GOSSIP_SMILEY_INDIFFERENT,  ":|",  0 },
-	{ GOSSIP_SMILEY_TOUNGE,       ":p",  0 },
-	{ GOSSIP_SMILEY_TOUNGE,       ":-p", 0 },
-	{ GOSSIP_SMILEY_TOUNGE,       ":P",  0 },
-	{ GOSSIP_SMILEY_TOUNGE,       ":-P", 0 },
-	{ GOSSIP_SMILEY_TOUNGE,       ";p",  0 },
-	{ GOSSIP_SMILEY_TOUNGE,       ";-p", 0 },
-	{ GOSSIP_SMILEY_TOUNGE,       ";P",  0 },
-	{ GOSSIP_SMILEY_TOUNGE,       ";-P", 0 },
-	{ GOSSIP_SMILEY_SHOCKED,      ":o",  0 },
-	{ GOSSIP_SMILEY_SHOCKED,      ":-o", 0 },
-	{ GOSSIP_SMILEY_SHOCKED,      ":O",  0 },
-	{ GOSSIP_SMILEY_SHOCKED,      ":-O", 0 },
-	{ GOSSIP_SMILEY_COOL,         "8)",  0 },
-	{ GOSSIP_SMILEY_COOL,         "B)",  0 },
-	{ GOSSIP_SMILEY_SORRY,        "*|",  0 },
-	{ GOSSIP_SMILEY_KISS,         ":*",  0 },
-	{ GOSSIP_SMILEY_SHUTUP,       ":#",  0 },
-	{ GOSSIP_SMILEY_SHUTUP,       ":-#", 0 },
-	{ GOSSIP_SMILEY_YAWN,         "|O",  0 },
-	{ GOSSIP_SMILEY_CONFUSED,     ":S",  0 },
-	{ GOSSIP_SMILEY_CONFUSED,     ":s",  0 },
-	{ GOSSIP_SMILEY_ANGEL,        "<)",  0 },
-	{ GOSSIP_SMILEY_OOOH,         ":x",  0 },
-	{ GOSSIP_SMILEY_LOOKAWAY,     "*)",  0 },
-	{ GOSSIP_SMILEY_LOOKAWAY,     "*-)", 0 },
-	{ GOSSIP_SMILEY_BLUSH,        "*S",  0 },
-	{ GOSSIP_SMILEY_BLUSH,        "*s",  0 },
-	{ GOSSIP_SMILEY_BLUSH,        "*$",  0 },
-	{ GOSSIP_SMILEY_COOLBIGSMILE, "8D",  0 },
-	{ GOSSIP_SMILEY_ANGRY,        ":@",  0 },
-	{ GOSSIP_SMILEY_BOSS,         "@)",  0 },
-	{ GOSSIP_SMILEY_MONKEY,       "#)",  0 },
-	{ GOSSIP_SMILEY_SILLY,        "O)",  0 },
-	{ GOSSIP_SMILEY_SICK,         "+o(", 0 },
-
-	/* backward smiley's */
-	{ GOSSIP_SMILEY_NORMAL,       "(:",  0 },
-	{ GOSSIP_SMILEY_WINK,         "(;",  0 },
-	{ GOSSIP_SMILEY_WINK,         "(-;", 0 },
-	{ GOSSIP_SMILEY_BIGEYE,       "(=",  0 },
-	{ GOSSIP_SMILEY_NOSE,         "(-:", 0 },
-	{ GOSSIP_SMILEY_CRY,          ")':", 0 },
-	{ GOSSIP_SMILEY_SAD,          "):",  0 },
-	{ GOSSIP_SMILEY_SAD,          ")-:", 0 },
-	{ GOSSIP_SMILEY_SCEPTICAL,    "/:",  0 },
-	{ GOSSIP_SMILEY_SCEPTICAL,    "//:", 0 },
-	{ GOSSIP_SMILEY_INDIFFERENT,  "|:",  0 },
-	{ GOSSIP_SMILEY_TOUNGE,       "d:",  0 },
-	{ GOSSIP_SMILEY_TOUNGE,       "d-:", 0 },
-	{ GOSSIP_SMILEY_TOUNGE,       "d;",  0 },
-	{ GOSSIP_SMILEY_TOUNGE,       "d-;", 0 },
-	{ GOSSIP_SMILEY_SHOCKED,      "o:",  0 },
-	{ GOSSIP_SMILEY_SHOCKED,      "O:",  0 },
-	{ GOSSIP_SMILEY_COOL,         "(8",  0 },
-	{ GOSSIP_SMILEY_COOL,         "(B",  0 },
-	{ GOSSIP_SMILEY_SORRY,        "|*",  0 },
-	{ GOSSIP_SMILEY_KISS,         "*:",  0 },
-	{ GOSSIP_SMILEY_SHUTUP,       "#:",  0 },
-	{ GOSSIP_SMILEY_SHUTUP,       "#-:", 0 },
-	{ GOSSIP_SMILEY_YAWN,         "O|",  0 },
-	{ GOSSIP_SMILEY_CONFUSED,     "S:",  0 },
-	{ GOSSIP_SMILEY_CONFUSED,     "s:",  0 },
-	{ GOSSIP_SMILEY_ANGEL,        "(>",  0 },
-	{ GOSSIP_SMILEY_OOOH,         "x:",  0 },
-	{ GOSSIP_SMILEY_LOOKAWAY,     "(*",  0 },
-	{ GOSSIP_SMILEY_LOOKAWAY,     "(-*", 0 },
-	{ GOSSIP_SMILEY_BLUSH,        "S*",  0 },
-	{ GOSSIP_SMILEY_BLUSH,        "s*",  0 },
-	{ GOSSIP_SMILEY_BLUSH,        "$*",  0 },
-	{ GOSSIP_SMILEY_ANGRY,        "@:",  0 },
-	{ GOSSIP_SMILEY_BOSS,         "(@",  0 },
-	{ GOSSIP_SMILEY_MONKEY,       "#)",  0 },
-	{ GOSSIP_SMILEY_SILLY,        "(O",  0 },
-	{ GOSSIP_SMILEY_SICK,         ")o+", 0 }
+static const GossipSmileyPattern smileys[] = {
+	/* Forward smileys. */
+	{ GOSSIP_SMILEY_NORMAL,       ":)"  },
+	{ GOSSIP_SMILEY_WINK,         ";)"  },
+	{ GOSSIP_SMILEY_WINK,         ";-)" },
+	{ GOSSIP_SMILEY_BIGEYE,       "=)"  },
+	{ GOSSIP_SMILEY_NOSE,         ":-)" },
+	{ GOSSIP_SMILEY_CRY,          ":'(" },
+	{ GOSSIP_SMILEY_SAD,          ":("  },
+	{ GOSSIP_SMILEY_SAD,          ":-(" },
+	{ GOSSIP_SMILEY_SCEPTICAL,    ":/"  },
+	{ GOSSIP_SMILEY_SCEPTICAL,    ":\\" },
+	{ GOSSIP_SMILEY_BIGSMILE,     ":D"  },
+	{ GOSSIP_SMILEY_BIGSMILE,     ":-D" },
+	{ GOSSIP_SMILEY_INDIFFERENT,  ":|"  },
+	{ GOSSIP_SMILEY_TOUNGE,       ":p"  },
+	{ GOSSIP_SMILEY_TOUNGE,       ":-p" },
+	{ GOSSIP_SMILEY_TOUNGE,       ":P"  },
+	{ GOSSIP_SMILEY_TOUNGE,       ":-P" },
+	{ GOSSIP_SMILEY_TOUNGE,       ";p"  },
+	{ GOSSIP_SMILEY_TOUNGE,       ";-p" },
+	{ GOSSIP_SMILEY_TOUNGE,       ";P"  },
+	{ GOSSIP_SMILEY_TOUNGE,       ";-P" },
+	{ GOSSIP_SMILEY_SHOCKED,      ":o"  },
+	{ GOSSIP_SMILEY_SHOCKED,      ":-o" },
+	{ GOSSIP_SMILEY_SHOCKED,      ":O"  },
+	{ GOSSIP_SMILEY_SHOCKED,      ":-O" },
+	{ GOSSIP_SMILEY_COOL,         "8)"  },
+	{ GOSSIP_SMILEY_COOL,         "B)"  },
+	{ GOSSIP_SMILEY_SORRY,        "*|"  },
+	{ GOSSIP_SMILEY_KISS,         ":*"  },
+	{ GOSSIP_SMILEY_SHUTUP,       ":#"  },
+	{ GOSSIP_SMILEY_SHUTUP,       ":-#" },
+	{ GOSSIP_SMILEY_YAWN,         "|O"  },
+	{ GOSSIP_SMILEY_CONFUSED,     ":S"  },
+	{ GOSSIP_SMILEY_CONFUSED,     ":s"  },
+	{ GOSSIP_SMILEY_ANGEL,        "<)"  },
+	{ GOSSIP_SMILEY_OOOH,         ":x"  },
+	{ GOSSIP_SMILEY_LOOKAWAY,     "*)"  },
+	{ GOSSIP_SMILEY_LOOKAWAY,     "*-)" },
+	{ GOSSIP_SMILEY_BLUSH,        "*S"  },
+	{ GOSSIP_SMILEY_BLUSH,        "*s"  },
+	{ GOSSIP_SMILEY_BLUSH,        "*$"  },
+	{ GOSSIP_SMILEY_COOLBIGSMILE, "8D"  },
+	{ GOSSIP_SMILEY_ANGRY,        ":@"  },
+	{ GOSSIP_SMILEY_BOSS,         "@)"  },
+	{ GOSSIP_SMILEY_MONKEY,       "#)"  },
+	{ GOSSIP_SMILEY_SILLY,        "O)"  },
+	{ GOSSIP_SMILEY_SICK,         "+o(" },
+	
+	/* Backward smileys. */
+	{ GOSSIP_SMILEY_NORMAL,       "(:"  },
+	{ GOSSIP_SMILEY_WINK,         "(;"  },
+	{ GOSSIP_SMILEY_WINK,         "(-;" },
+	{ GOSSIP_SMILEY_BIGEYE,       "(="  },
+	{ GOSSIP_SMILEY_NOSE,         "(-:" },
+	{ GOSSIP_SMILEY_CRY,          ")':" },
+	{ GOSSIP_SMILEY_SAD,          "):"  },
+	{ GOSSIP_SMILEY_SAD,          ")-:" },
+	{ GOSSIP_SMILEY_SCEPTICAL,    "/:"  },
+	{ GOSSIP_SMILEY_SCEPTICAL,    "//:" },
+	{ GOSSIP_SMILEY_INDIFFERENT,  "|:"  },
+	{ GOSSIP_SMILEY_TOUNGE,       "d:"  },
+	{ GOSSIP_SMILEY_TOUNGE,       "d-:" },
+	{ GOSSIP_SMILEY_TOUNGE,       "d;"  },
+	{ GOSSIP_SMILEY_TOUNGE,       "d-;" },
+	{ GOSSIP_SMILEY_SHOCKED,      "o:"  },
+	{ GOSSIP_SMILEY_SHOCKED,      "O:"  },
+	{ GOSSIP_SMILEY_COOL,         "(8"  },
+	{ GOSSIP_SMILEY_COOL,         "(B"  },
+	{ GOSSIP_SMILEY_SORRY,        "|*"  },
+	{ GOSSIP_SMILEY_KISS,         "*:"  },
+	{ GOSSIP_SMILEY_SHUTUP,       "#:"  },
+	{ GOSSIP_SMILEY_SHUTUP,       "#-:" },
+	{ GOSSIP_SMILEY_YAWN,         "O|"  },
+	{ GOSSIP_SMILEY_CONFUSED,     "S:"  },
+	{ GOSSIP_SMILEY_CONFUSED,     "s:"  },
+	{ GOSSIP_SMILEY_ANGEL,        "(>"  },
+	{ GOSSIP_SMILEY_OOOH,         "x:"  },
+	{ GOSSIP_SMILEY_LOOKAWAY,     "(*"  },
+	{ GOSSIP_SMILEY_LOOKAWAY,     "(-*" },
+	{ GOSSIP_SMILEY_BLUSH,        "S*"  },
+	{ GOSSIP_SMILEY_BLUSH,        "s*"  },
+	{ GOSSIP_SMILEY_BLUSH,        "$*"  },
+	{ GOSSIP_SMILEY_ANGRY,        "@:"  },
+	{ GOSSIP_SMILEY_BOSS,         "(@"  },
+	{ GOSSIP_SMILEY_MONKEY,       "#)"  },
+	{ GOSSIP_SMILEY_SILLY,        "(O"  },
+	{ GOSSIP_SMILEY_SICK,         ")o+" }
 };
 
 static void     gossip_chat_view_class_init          (GossipChatViewClass      *klass);
@@ -191,15 +189,12 @@ static gboolean chat_view_drag_motion                (GtkWidget                *
 static void     chat_view_size_allocate              (GtkWidget                *widget,
 						      GtkAllocation            *alloc);
 static void     chat_view_setup_tags                 (GossipChatView           *view);
-static void     chat_view_system_font_update         (GossipChatView           *view,
-						      GConfClient              *gconf_client);
-static void     chat_view_system_font_notify_cb      (GConfClient              *gconf_client,
-						      guint                     id,
-						      GConfEntry               *entry,
+static void     chat_view_system_font_update         (GossipChatView           *view);
+static void     chat_view_notify_system_font_cb      (GossipConf               *conf,
+						      const gchar              *key,
 						      gpointer                  user_data);
-static void     chat_view_show_avatars_notify_cb     (GConfClient              *gconf_client,
-						      guint                     id,
-						      GConfEntry               *entry,
+static void     chat_view_notify_show_avatars_cb     (GossipConf               *conf,
+						      const gchar              *key,
 						      gpointer                  user_data);
 static void     chat_view_populate_popup             (GossipChatView           *view,
 						      GtkMenu                  *menu,
@@ -212,7 +207,6 @@ static gboolean chat_view_url_event_cb               (GtkTextTag               *
 						      GdkEvent                 *event,
 						      GtkTextIter              *iter,
 						      GtkTextBuffer            *buffer);
-static void     chat_view_open_address               (const gchar              *url);
 static void     chat_view_open_address_cb            (GtkMenuItem              *menuitem,
 						      const gchar              *url);
 static void     chat_view_copy_address_cb            (GtkMenuItem              *menuitem,
@@ -280,8 +274,8 @@ static void
 gossip_chat_view_init (GossipChatView *view)
 {
 	GossipChatViewPriv *priv;
-	GConfClient        *gconf_client;
-
+	gboolean            show_avatars;
+		
 	priv = GET_PRIV (view);
 
 	priv->buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
@@ -291,37 +285,39 @@ gossip_chat_view_init (GossipChatView *view)
 
 	priv->allow_scrolling = TRUE;
 
+	priv->is_group_chat = FALSE;
+
 	g_object_set (view,
 		      "wrap-mode", GTK_WRAP_WORD_CHAR,
 		      "editable", FALSE,
 		      "cursor-visible", FALSE,
 		      NULL);
 
-	gconf_client = gossip_app_get_gconf_client ();
-	
 	priv->notify_system_fonts_id = 
-		gconf_client_notify_add (gconf_client,
+		gossip_conf_notify_add (gossip_conf_get (),
 					 "/desktop/gnome/interface/document_font_name",
-					 chat_view_system_font_notify_cb,
-					 view, NULL, NULL);
-	chat_view_system_font_update (view, gconf_client);
+					 chat_view_notify_system_font_cb,
+					 view);
+	chat_view_system_font_update (view);
 	
 	priv->notify_show_avatars_id = 
-		gconf_client_notify_add (gconf_client,
-					 GCONF_UI_SHOW_AVATARS,
-					 chat_view_show_avatars_notify_cb,
-					 view, NULL, NULL);
+		gossip_conf_notify_add (gossip_conf_get (),
+					 GOSSIP_PREFS_UI_SHOW_AVATARS,
+					 chat_view_notify_show_avatars_cb,
+					 view);
 
 	chat_view_setup_tags (view);
 
-	gossip_theme_manager_apply (gossip_theme_manager_get (), view);
-	gossip_theme_manager_update_show_avatars (gossip_theme_manager_get (),
-						  view,
-						  gconf_client_get_bool (
-							  gconf_client,
-							  GCONF_UI_SHOW_AVATARS,
-							  NULL));
+	gossip_theme_manager_apply_saved (gossip_theme_manager_get (), view);
 
+	show_avatars = FALSE;
+	gossip_conf_get_bool (gossip_conf_get (),
+			       GOSSIP_PREFS_UI_SHOW_AVATARS,
+			       &show_avatars);
+	
+	gossip_theme_manager_update_show_avatars (gossip_theme_manager_get (),
+						  view, show_avatars);
+	
 	g_signal_connect (view,
 			  "populate-popup",
 			  G_CALLBACK (chat_view_populate_popup),
@@ -339,14 +335,16 @@ chat_view_finalize (GObject *object)
 {
 	GossipChatView     *view;
 	GossipChatViewPriv *priv;
-	GConfClient        *gconf_client;
 
 	view = GOSSIP_CHAT_VIEW (object);
 	priv = GET_PRIV (view);
 
-	gconf_client = gossip_app_get_gconf_client ();
-	gconf_client_notify_remove (gconf_client, priv->notify_system_fonts_id);
-	gconf_client_notify_remove (gconf_client, priv->notify_show_avatars_id);
+	gossip_conf_notify_remove (gossip_conf_get (), priv->notify_system_fonts_id);
+	gossip_conf_notify_remove (gossip_conf_get (), priv->notify_show_avatars_id);
+
+	if (priv->last_contact) {
+		g_object_unref (priv->last_contact);
+	}
 
 	G_OBJECT_CLASS (gossip_chat_view_parent_class)->finalize (object);
 }
@@ -416,17 +414,14 @@ chat_view_setup_tags (GossipChatView *view)
 }
 
 static void
-chat_view_system_font_update (GossipChatView *view,
-			      GConfClient    *gconf_client)
+chat_view_system_font_update (GossipChatView *view)
 {
 	PangoFontDescription *font_description = NULL;
 	gchar                *font_name;
 
-	font_name = gconf_client_get_string (gconf_client, 
-					     "/desktop/gnome/interface/document_font_name", 
-					     NULL);
-	
-	if (font_name) {
+	if (gossip_conf_get_string (gossip_conf_get (),
+				     "/desktop/gnome/interface/document_font_name",
+				     &font_name) && font_name) {
 		font_description = pango_font_description_from_string (font_name);
 		g_free (font_name);
 	} else {
@@ -441,46 +436,45 @@ chat_view_system_font_update (GossipChatView *view,
 }
 
 static void
-chat_view_system_font_notify_cb (GConfClient    *gconf_client,
-				 guint           id,
-				 GConfEntry     *entry,
-				 gpointer        user_data)
+chat_view_notify_system_font_cb (GossipConf  *conf,
+				 const gchar *key,
+				 gpointer     user_data)
 {
 	GossipChatView *view;
+	gboolean        show_avatars = FALSE;
 
 	view = user_data;
-	chat_view_system_font_update (view, gconf_client);
+
+	chat_view_system_font_update (view);
 	
 	/* Ugly, again, to adjust the vertical position of the nick... Will fix
 	 * this when reworking the theme manager so that view register
 	 * themselves with it instead of the other way around.
 	 */
+	gossip_conf_get_bool (conf,
+			       GOSSIP_PREFS_UI_SHOW_AVATARS,
+			       &show_avatars);
+	
 	gossip_theme_manager_update_show_avatars (gossip_theme_manager_get (),
-						  view,
-						  gconf_client_get_bool (
-							  gconf_client,
-							  GCONF_UI_SHOW_AVATARS,
-							  NULL));
+						  view, show_avatars);
 }
 
 static void
-chat_view_show_avatars_notify_cb (GConfClient    *gconf_client,
-				  guint           id,
-				  GConfEntry     *entry,
-				  gpointer        user_data)
+chat_view_notify_show_avatars_cb (GossipConf  *conf,
+				  const gchar *key,
+				  gpointer     user_data)
 {
 	GossipChatView     *view;
 	GossipChatViewPriv *priv;
-	GConfValue         *value;
+	gboolean            show_avatars = FALSE;
 
 	view = user_data;
 	priv = GET_PRIV (view);
 
-	value = gconf_entry_get_value (entry);
+	gossip_conf_get_bool (conf, key, &show_avatars);
 	
 	gossip_theme_manager_update_show_avatars (gossip_theme_manager_get (),
-						  view,
-						  gconf_value_get_bool (value));
+						  view, show_avatars);
 }
 
 static void
@@ -639,7 +633,7 @@ chat_view_url_event_cb (GtkTextTag    *tag,
 							&end,
 							FALSE);
 
-			chat_view_open_address (str);
+			gossip_url_show (str);
 			g_free (str);
 		}
 	}
@@ -647,52 +641,10 @@ chat_view_url_event_cb (GtkTextTag    *tag,
 	return FALSE;
 }
 
-static void
-chat_view_open_address (const gchar *url)
-{
-	gchar *real_url;
-
-	/* The URL opening code can't handle schemeless strings, so we try to be
-	 * smart and add http if there is no scheme or doesn't look like a mail
-	 * address. This should work in most cases, and let us click on strings
-	 * like "www.gnome.org".
-	 */
-	if (!g_str_has_prefix (url, "http://") &&
-	    !strstr (url, ":/") &&
-	    !strstr (url, "@")) {
-		real_url = g_strdup_printf ("http://%s", url);
-	} else {
-		real_url = g_strdup (url);
-	}
-
-#ifdef USE_GNOMEVFS_FOR_URL
-	{
-		GnomeVFSResult result;
-		
-		result = gnome_vfs_url_show (real_url);
-		if (result != GNOME_VFS_OK) {
-			g_warning ("Couldn't show URL:'%s'", real_url);
-		}
-	}
-#else
-	{
-		GError *error = NULL;
-		
-		gnome_url_show (real_url, &error);
-		if (error) {
-			g_warning ("Couldn't show URL:'%s'", real_url);
-			g_error_free (error);
-		}
-	}
-#endif
-
-	g_free (real_url);
-}
-
 static void
 chat_view_open_address_cb (GtkMenuItem *menuitem, const gchar *url)
 {
-	chat_view_open_address (url);
+	gossip_url_show (url);
 }
 
 static void
@@ -723,12 +675,11 @@ chat_view_insert_text_with_emoticons (GtkTextBuffer *buf,
 	gint         i;
 	gint         match;
 	gint         submatch;
-	gboolean     use_smileys;
+	gboolean     use_smileys = FALSE;
 
-	use_smileys = gconf_client_get_bool (
-		gossip_app_get_gconf_client (),
-		"/apps/gossip/conversation/graphical_smileys",
-		NULL);
+	gossip_conf_get_bool (gossip_conf_get (),
+			       GOSSIP_PREFS_CHAT_SHOW_SMILEYS,
+			       &use_smileys);
 
 	if (!use_smileys) {
 		gtk_text_buffer_insert (buf, iter, str, -1);
@@ -736,13 +687,12 @@ chat_view_insert_text_with_emoticons (GtkTextBuffer *buf,
 	}
 
 	while (*str) {
+		gint         smileys_index[G_N_ELEMENTS (smileys)];
 		GdkPixbuf   *pixbuf;
 		gint         len;
 		const gchar *start;
 
-		for (i = 0; i < G_N_ELEMENTS (smileys); i++) {
-			smileys[i].index = 0;
-		}
+		memset (smileys_index, 0, sizeof (smileys_index));
 
 		match = -1;
 		submatch = -1;
@@ -762,15 +712,23 @@ chat_view_insert_text_with_emoticons (GtkTextBuffer *buf,
 				submatch = -1;
 
 				for (i = 0; i < G_N_ELEMENTS (smileys); i++) {
-					if (smileys[i].pattern[smileys[i].index] == c) {
+					/* Only try to match if we already have
+					 * a beginning match for the pattern, or
+					 * if it's the first character in the
+					 * pattern, if it's not in the middle of
+					 * a word.
+					 */
+					if (((smileys_index[i] == 0 && (prev_c == 0 || g_unichar_isspace (prev_c))) ||
+					     smileys_index[i] > 0) &&
+					    smileys[i].pattern[smileys_index[i]] == c) {
 						submatch = i;
-
-						smileys[i].index++;
-						if (!smileys[i].pattern[smileys[i].index]) {
+						
+						smileys_index[i]++;
+						if (!smileys[i].pattern[smileys_index[i]]) {
 							match = i;
 						}
 					} else {
-						smileys[i].index = 0;
+						smileys_index[i] = 0;
 					}
 				}
 			}
@@ -924,7 +882,7 @@ chat_view_maybe_append_date_and_time (GossipChatView *view,
 	
 	if (append_date) {
 		gchar buf[256];
-		
+
 		g_date_strftime (buf, 256, _("%A %d %B %Y"), date);
 		g_string_append (str, buf);
 
@@ -939,7 +897,7 @@ chat_view_maybe_append_date_and_time (GossipChatView *view,
 	if (append_time) {
 		gchar *tmp;
 
-		tmp = gossip_time_to_timestamp (timestamp);
+		tmp = gossip_time_to_string_local (timestamp, GOSSIP_TIME_FORMAT_DISPLAY_SHORT);
 		g_string_append (str, tmp);
 		g_free (tmp);
 	}
@@ -1465,22 +1423,28 @@ chat_view_theme_changed_cb (GossipThemeManager *manager,
 			    GossipChatView     *view)
 {
 	GossipChatViewPriv *priv;
-	
+	gboolean            show_avatars = FALSE;
+
 	priv = GET_PRIV (view);
-	
+
+	/* Do nothing for group chats since they are always the same
+	 * theme.
+	 */
+	if (priv->is_group_chat) {
+		return;
+	}
+
 	priv->last_block_type = BLOCK_TYPE_NONE;
 	
-	gossip_theme_manager_apply (manager, view);
+	gossip_theme_manager_apply_saved (manager, view);
 
 	/* Needed for now to update the "rise" property of the names to get it
 	 * vertically centered.
 	 */
-	gossip_theme_manager_update_show_avatars (manager,
-						  view,
-						  gconf_client_get_bool (
-							  gossip_app_get_gconf_client (),
-							  GCONF_UI_SHOW_AVATARS,
-							  NULL));
+	gossip_conf_get_bool (gossip_conf_get (),
+			       GOSSIP_PREFS_UI_SHOW_AVATARS,
+			       &show_avatars);
+	gossip_theme_manager_update_show_avatars (manager, view, show_avatars);
 }
 
 GossipChatView *
@@ -1832,6 +1796,8 @@ gossip_chat_view_scroll_down (GossipChatView *view)
 				      FALSE,
 				      0,
 				      0);
+
+	gtk_text_buffer_delete_mark (buffer, mark);
 }
 
 gboolean
@@ -1874,35 +1840,55 @@ gossip_chat_view_find (GossipChatView *view,
 		       const gchar    *search_criteria,
 		       gboolean        new_search)
 {
+	GossipChatViewPriv *priv;
 	GtkTextBuffer      *buffer;
 	GtkTextIter         iter_at_mark;
 	GtkTextIter         iter_match_start;
 	GtkTextIter         iter_match_end;
-	static GtkTextMark *find_mark = NULL;
-	static gboolean     wrapped = FALSE;
 	gboolean            found;
 	gboolean            from_start = FALSE;
 
 	g_return_if_fail (GOSSIP_IS_CHAT_VIEW (view));
 	g_return_if_fail (search_criteria != NULL);
-	g_return_if_fail (strlen (search_criteria) > 0);
 
-	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
-    
+	priv = GET_PRIV (view);
+	
+	buffer = priv->buffer;
+	
+	if (strlen (search_criteria) == 0) {
+		if (priv->find_mark) {
+			gtk_text_buffer_get_start_iter (buffer, &iter_at_mark);
+			
+			gtk_text_buffer_move_mark (buffer,
+						   priv->find_mark,
+						   &iter_at_mark);
+			gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (view),
+						      priv->find_mark,
+						      0.0,
+						      TRUE,
+						      0.0,
+						      0.0);
+			gtk_text_buffer_select_range (buffer,
+						      &iter_at_mark,
+						      &iter_at_mark);
+		}
+		
+		return;
+	}
+	
 	if (new_search) {
-		find_mark = NULL;
 		from_start = TRUE;
 	}
      
-	if (find_mark) {
-		gtk_text_buffer_get_iter_at_mark (buffer, &iter_at_mark, find_mark);
-		gtk_text_buffer_delete_mark (buffer, find_mark); 
-		find_mark = NULL;
+	if (priv->find_mark) {
+		gtk_text_buffer_get_iter_at_mark (buffer,
+						  &iter_at_mark,
+						  priv->find_mark);
 	} else {
 		gtk_text_buffer_get_start_iter (buffer, &iter_at_mark);
 		from_start = TRUE;
 	}
-
+	
 	found = gossip_text_iter_forward_search (&iter_at_mark,
 						 search_criteria,
 						 &iter_match_start, 
@@ -1915,26 +1901,35 @@ gossip_chat_view_find (GossipChatView *view,
 		}
 	
 		/* Here we wrap around. */
-		if (!new_search && !wrapped) {
-			wrapped = TRUE;
+		if (!new_search && !priv->find_wrapped) {
+			priv->find_wrapped = TRUE;
 			gossip_chat_view_find (view, search_criteria, FALSE);
-			wrapped = FALSE;
+			priv->find_wrapped = FALSE;
 		}
 
 		return;
 	}
     
 	/* Set new mark and show on screen */
-	find_mark = gtk_text_buffer_create_mark (buffer, NULL, &iter_match_end, TRUE); 
+	if (!priv->find_mark) {
+		priv->find_mark = gtk_text_buffer_create_mark (buffer, NULL,
+							       &iter_match_end,
+							       TRUE);
+	} else {
+		gtk_text_buffer_move_mark (buffer,
+					   priv->find_mark,
+					   &iter_match_end);
+	}
+	
 	gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (view),
-				      find_mark,
+				      priv->find_mark,
 				      0.0,
 				      TRUE,
 				      0.5,
 				      0.5);
 
 	gtk_text_buffer_move_mark_by_name (buffer, "selection_bound", &iter_match_start);
-	gtk_text_buffer_move_mark_by_name (buffer, "insert", &iter_match_end);	
+	gtk_text_buffer_move_mark_by_name (buffer, "insert", &iter_match_end);
 }
 
 void
@@ -1964,7 +1959,7 @@ gossip_chat_view_highlight (GossipChatView *view,
 		return;
 	}
 
-	while (TRUE) {
+	while (1) {
 		found = gossip_text_iter_forward_search (&iter, 
 							 text, 
 							 &iter_match_start, 
@@ -2128,3 +2123,24 @@ gossip_chat_view_get_smiley_menu (GCallback    callback,
 	return menu;
 }
 
+void
+gossip_chat_view_set_is_group_chat (GossipChatView *view,
+				    gboolean        is_group_chat)
+{
+	GossipChatViewPriv *priv;
+
+	g_return_if_fail (GOSSIP_IS_CHAT_VIEW (view));
+	
+	priv = GET_PRIV (view);
+
+	priv->is_group_chat = is_group_chat;
+
+	if (is_group_chat) {
+		gossip_theme_manager_apply (gossip_theme_manager_get (), 
+					    view, 
+					    NULL);
+	} else {
+		gossip_theme_manager_apply_saved (gossip_theme_manager_get (), 
+						  view);
+	}
+}
