@@ -21,13 +21,13 @@
 
 #include <config.h>
 #include <string.h>
-
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <glade/glade.h>
 #include <glib/gi18n.h>
-#include <gconf/gconf-client.h>
 
+#include <libgossip/gossip-debug.h>
+#include <libgossip/gossip-conf.h>
 #include <libgossip/gossip-protocol.h>
 
 #include "gossip-add-contact-window.h"
@@ -46,8 +46,7 @@
 
 #define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_CHAT_WINDOW, GossipChatWindowPriv))
 
-#define DEBUG_MSG(x)
-/* #define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n");  */
+#define DEBUG_DOMAIN "ChatWindow"
 
 static void       gossip_chat_window_class_init         (GossipChatWindowClass *klass);
 static void       gossip_chat_window_init               (GossipChatWindow      *window);
@@ -200,17 +199,12 @@ struct _GossipChatWindowPriv {
 	GtkWidget   *menu_tabs_right;
 	GtkWidget   *menu_tabs_detach;
 
-	/* Geometry information */
 	guint        save_geometry_id;
-	gint         last_x;
-	gint         last_y;
-	gint         last_w;
-	gint         last_h;
 };
 
 static GList *chat_windows = NULL;
 
-static guint tab_accel_keys[] = {
+static const guint tab_accel_keys[] = {
 	GDK_1, GDK_2, GDK_3, GDK_4, GDK_5,
 	GDK_6, GDK_7, GDK_8, GDK_9, GDK_0
 };
@@ -219,7 +213,7 @@ enum DndDropType {
 	DND_DROP_TYPE_CONTACT_ID,
 };
 
-static GtkTargetEntry drop_types[] = {
+static const GtkTargetEntry drop_types[] = {
 	{ "text/contact-id", 0, DND_DROP_TYPE_CONTACT_ID },
 };
 
@@ -264,7 +258,7 @@ gossip_chat_window_init (GossipChatWindow *window)
 	priv->tooltips = g_object_ref (gtk_tooltips_new ());
 	gtk_object_sink (GTK_OBJECT (priv->tooltips));
 	
-	glade = gossip_glade_get_file (GLADEDIR "/chat.glade",
+	glade = gossip_glade_get_file ("chat.glade",
 				       "chat_window",
 				       NULL,
 				       "chat_window", &priv->dialog,
@@ -327,6 +321,8 @@ gossip_chat_window_init (GossipChatWindow *window)
 					 closure);
 	}
 
+	g_object_unref (accel_group);
+
 	/* Set the contact information menu item image to the Gossip
 	 * stock image 
 	 */ 
@@ -422,15 +418,13 @@ gossip_chat_window_init (GossipChatWindow *window)
 GossipChatWindow *
 gossip_chat_window_get_default (void)
 {
-	GList       *l;
-        GConfClient *gconf_client;
-        gboolean     separate_windows;
-        
-	gconf_client = gossip_app_get_gconf_client ();
-        separate_windows = gconf_client_get_bool (gconf_client,
-                                                  GCONF_UI_SEPARATE_CHAT_WINDOWS,
-                                                  NULL);
+	GList    *l;
+        gboolean  separate_windows = TRUE;
 
+	gossip_conf_get_bool (gossip_conf_get (),
+			      GOSSIP_PREFS_UI_SEPARATE_CHAT_WINDOWS,
+			      &separate_windows);
+	
         if (separate_windows) {
                 /* Always create a new window */
                 return NULL;
@@ -1005,12 +999,14 @@ static gboolean
 chat_window_save_geometry_timeout_cb (GossipChatWindow *window)
 {
 	GossipChatWindowPriv *priv;
+	gint                  x, y, w, h;
 
 	priv = GET_PRIV (window);
 
-	gossip_chat_save_geometry (priv->current_chat, 
-				   priv->last_x, priv->last_y, 
-				   priv->last_w, priv->last_h);
+	gtk_window_get_size (GTK_WINDOW (priv->dialog), &w, &h);
+	gtk_window_get_position (GTK_WINDOW (priv->dialog), &x, &y);     
+
+	gossip_chat_save_geometry (priv->current_chat, x, y, w, h);
 
 	priv->save_geometry_id = 0;
 
@@ -1026,14 +1022,13 @@ chat_window_configure_event_cb (GtkWidget         *widget,
 
 	priv = GET_PRIV (window);
 
-	/* Only save geometry information if there is ONE chat visible */
+	/* Only save geometry information if there is ONE chat visible. */
 	if (g_list_length (priv->chats) > 1) {
 		return FALSE;
 	}
 
 	if (priv->save_geometry_id != 0) {
 		g_source_remove (priv->save_geometry_id);
-		priv->save_geometry_id = 0;
 	}
 
 	priv->save_geometry_id = 
@@ -1041,11 +1036,6 @@ chat_window_configure_event_cb (GtkWidget         *widget,
 			       (GSourceFunc) chat_window_save_geometry_timeout_cb,
 			       window);
 
-	priv->last_x = event->x;
-	priv->last_y = event->y;
-	priv->last_w = event->width;
-	priv->last_h = event->height;
-
 	return FALSE;
 }
 
@@ -1298,7 +1288,7 @@ chat_window_delete_event_cb (GtkWidget        *dialog,
 
 	priv = GET_PRIV (window);
 
-	DEBUG_MSG (("ChatWindow: Delete event received"));
+	gossip_debug (DEBUG_DOMAIN, "Delete event received");
 
 	list = g_list_copy (priv->chats);
 
@@ -1387,23 +1377,28 @@ chat_window_new_message_cb (GossipChat       *chat,
 
 	priv = GET_PRIV (window);
 
-	DEBUG_MSG (("ChatWindow: New message, do we have focus?"));
+	gossip_request_user_attention ();
+	
 	if (!gossip_chat_window_has_focus (window)) {
 		GossipContact *own_contact;
 
+		gossip_debug (DEBUG_DOMAIN, "New message, no focus");
+
 		priv->new_msg = TRUE;
 		chat_window_update_title (window, chat);
 
 		if (gossip_chat_is_group_chat (chat)) {
 			own_contact = gossip_chat_get_own_contact (chat);
 
-			DEBUG_MSG (("ChatWindow: Should we highlight this nick?"));
 			if (gossip_chat_should_highlight_nick (message, own_contact)) {
+				gossip_debug (DEBUG_DOMAIN, "Highlight this nick");
 				chat_window_set_urgency_hint (window, TRUE);
 			}
 		} else {
 			chat_window_set_urgency_hint (window, TRUE);
 		}
+	} else {
+		gossip_debug (DEBUG_DOMAIN, "New message, we have focus");
 	}
 
 	if (chat == priv->current_chat) {
@@ -1449,7 +1444,7 @@ chat_window_switch_page_cb (GtkNotebook	     *notebook,
 	GossipChat           *chat;
 	GtkWidget            *child;
 
- 	DEBUG_MSG (("ChatWindow: Tab switched"));
+ 	gossip_debug (DEBUG_DOMAIN, "Tab switched");
 
 	priv = GET_PRIV (window);
 
@@ -1478,7 +1473,7 @@ chat_window_tab_added_cb (GossipNotebook   *notebook,
 	GtkWidget            *label;
 	gboolean              expand = TRUE;
 
- 	DEBUG_MSG (("ChatWindow: Tab added"));
+ 	gossip_debug (DEBUG_DOMAIN, "Tab added");
 
 	priv = GET_PRIV (window);
 
@@ -1536,7 +1531,7 @@ chat_window_tab_removed_cb (GossipNotebook   *notebook,
 	GossipChatWindowPriv *priv;
 	GossipChat           *chat;
 
-	DEBUG_MSG (("ChatWindow: Tab removed"));
+	gossip_debug (DEBUG_DOMAIN, "Tab removed");
 
 	priv = GET_PRIV (window);
 
@@ -1584,7 +1579,7 @@ chat_window_tab_detached_cb (GossipNotebook   *notebook,
 	GossipChat	     *chat;
 	gint                  x, y;
 
-	DEBUG_MSG (("ChatWindow: Tab detached"));
+	gossip_debug (DEBUG_DOMAIN, "Tab detached");
 
 	chat = g_object_get_data (G_OBJECT (child), "chat");
 	new_window = gossip_chat_window_new ();
@@ -1609,7 +1604,7 @@ static void
 chat_window_tabs_reordered_cb (GossipNotebook   *notebook,
 			       GossipChatWindow *window)
 {
- 	DEBUG_MSG (("ChatWindow: Tabs reordered"));
+ 	gossip_debug (DEBUG_DOMAIN, "Tabs reordered");
 
 	chat_window_update_menu (window);
 }
@@ -1647,15 +1642,15 @@ chat_window_drag_data_received (GtkWidget        *widget,
 	const gchar       *id;
 
 	id = (const gchar*) selection->data;
-	DEBUG_MSG (("ChatWindow: Received drag & drop contact "
-		    "from roster with id:'%s'", 
-		    id));
+	gossip_debug (DEBUG_DOMAIN, "Received drag & drop contact "
+		      "from roster with id:'%s'", 
+		      id);
 
 	contact = gossip_session_find_contact (gossip_app_get_session (), id);
 	
 	if (!contact) {
-		DEBUG_MSG (("ChatWindow: No contact found associated "
-			    "with drag & drop"));
+		gossip_debug (DEBUG_DOMAIN, "No contact found associated "
+			      "with drag & drop");
 		return;
 	}
 
@@ -1687,7 +1682,7 @@ chat_window_urgency_timeout_func (GossipChatWindow *window)
 
 	priv = GET_PRIV (window);
 
-	DEBUG_MSG (("ChatWindow: Turning off urgency hint"));
+	gossip_debug (DEBUG_DOMAIN, "Turning off urgency hint");
 	gtk_window_set_urgency_hint (GTK_WINDOW (priv->dialog), FALSE);
 
 	priv->urgency_timeout_id = 0;
@@ -1706,19 +1701,19 @@ chat_window_set_urgency_hint (GossipChatWindow *window,
 	if (!urgent) {
 		/* Remove any existing hint and timeout. */
 		if (priv->urgency_timeout_id) {
-			DEBUG_MSG (("ChatWindow: Turning off urgency hint"));
+			gossip_debug (DEBUG_DOMAIN, "Turning off urgency hint");
 			gtk_window_set_urgency_hint (GTK_WINDOW (priv->dialog), FALSE);
 			g_source_remove (priv->urgency_timeout_id);
 			priv->urgency_timeout_id = 0;
 		}
 		return;
 	}	
-	
+
 	/* Add a new hint and renew any exising timeout or add a new one. */
 	if (priv->urgency_timeout_id) {
 		g_source_remove (priv->urgency_timeout_id);
 	} else {
-		DEBUG_MSG (("ChatWindow: Turning on urgency hint"));
+		gossip_debug (DEBUG_DOMAIN, "Turning on urgency hint");
 		gtk_window_set_urgency_hint (GTK_WINDOW (priv->dialog), TRUE);
 	}
 
@@ -1765,7 +1760,7 @@ gossip_chat_window_add_chat (GossipChatWindow *window,
 	GossipChatWindowPriv *priv;
 	GtkWidget            *label;
 
-	DEBUG_MSG (("ChatWindow: Adding chat"));
+	gossip_debug (DEBUG_DOMAIN, "Adding chat");
 
 	priv = GET_PRIV (window);
 	
@@ -1807,7 +1802,7 @@ gossip_chat_window_remove_chat (GossipChatWindow *window,
 {
 	GossipChatWindowPriv *priv;
 
-	DEBUG_MSG (("ChatWindow: Removing chat"));
+	gossip_debug (DEBUG_DOMAIN, "Removing chat");
 
 	priv = GET_PRIV (window);
 
