@@ -23,13 +23,14 @@
 #include <glade/glade.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
-#include <libgnomeui/gnome-href.h>
 
 #include <libgossip/gossip-contact.h>
 #include <libgossip/gossip-session.h>
+#include <libgossip/gossip-paths.h>
 
 #include "gossip-app.h"
 #include "gossip-avatar-image.h"
+#include "gossip-ui-utils.h"
 #include "gossip-contact-info-dialog.h"
 
 typedef struct {
@@ -313,8 +314,7 @@ contact_info_dialog_get_vcard_cb (GossipResult   result,
 		show_personal = TRUE;
 
 		link = g_strdup_printf ("mailto:%s", str);
-		
-		href = gnome_href_new (link, str);
+		href = gossip_link_button_new (link, str);
 
 		alignment = gtk_alignment_new (0, 1, 0, 0.5);
 		gtk_container_add (GTK_CONTAINER (alignment), href);
@@ -340,7 +340,7 @@ contact_info_dialog_get_vcard_cb (GossipResult   result,
 
 		show_personal = TRUE;
 
-		href = gnome_href_new (str, str);
+		href = gossip_link_button_new (str, str);
 
 		alignment = gtk_alignment_new (0, 1, 0, 0.5);
 		gtk_container_add (GTK_CONTAINER (alignment), href);
@@ -490,6 +490,8 @@ contact_info_dialog_subscribe_clicked_cb (GtkWidget               *widget,
 				    gossip_contact_get_name (dialog->contact),
 				    NULL, /* group */
 				    message);
+
+	g_object_unref (account);
 }
 
 static void
@@ -545,6 +547,7 @@ gossip_contact_info_dialog_show (GossipContact *contact)
 	guint                    id;
 	GtkWidget               *avatar_image_placeholder;
 	GdkPixbuf               *pixbuf;
+	gchar                   *path;
 
 	contact_info_dialog_init ();
 
@@ -563,7 +566,7 @@ gossip_contact_info_dialog_show (GossipContact *contact)
 
 	g_hash_table_insert (contact_info_dialogs, dialog->contact, dialog);
 
-	glade = gossip_glade_get_file (GLADEDIR "/main.glade",
+	glade = gossip_glade_get_file ("main.glade",
 				       "contact_information_dialog",
 				       NULL,
 				       "contact_information_dialog", &dialog->dialog,
@@ -598,7 +601,9 @@ gossip_contact_info_dialog_show (GossipContact *contact)
 	gtk_container_add (GTK_CONTAINER (avatar_image_placeholder),
 			   dialog->avatar_image);
 
-	pixbuf = gdk_pixbuf_new_from_file (IMAGEDIR "/vcard_48.png", NULL);
+	path = gossip_paths_get_image_path ("vcard_48.png");
+	pixbuf = gdk_pixbuf_new_from_file (path, NULL);
+	g_free (path);
 	if (pixbuf) {
 		gossip_avatar_image_set_pixbuf (
 			GOSSIP_AVATAR_IMAGE (dialog->avatar_image), pixbuf);
@@ -678,5 +683,7 @@ gossip_contact_info_dialog_show (GossipContact *contact)
 		g_timeout_add (500,
 			       (GSourceFunc) contact_info_dialog_show_timeout_cb,
 			       dialog);
+
+	g_object_unref (account);
 }
 
