@@ -324,7 +324,7 @@ enum {
 
 #define GIF_CB(x)    ((GtkItemFactoryCallback)(x))
 
-static GtkItemFactoryEntry item_menu_items[] = {
+static const GtkItemFactoryEntry item_menu_items[] = {
 	{ N_("/Contact Infor_mation"),
 	  NULL,
 	  GIF_CB (contact_list_item_menu_info_cb),
@@ -393,7 +393,7 @@ static GtkItemFactoryEntry item_menu_items[] = {
 	  GTK_STOCK_JUSTIFY_LEFT }
 };
 
-static GtkItemFactoryEntry group_menu_items[] = {
+static const GtkItemFactoryEntry group_menu_items[] = {
 	{ N_("/Re_name group"),
 	  NULL,
 	  GIF_CB (contact_list_group_menu_rename_cb),
@@ -408,14 +408,14 @@ enum DndDragType {
 	DND_DRAG_TYPE_CONTACT_ID,
 };
 
-static GtkTargetEntry drag_types_dest[] = {
+static const GtkTargetEntry drag_types_dest[] = {
 	{ "STRING",          0, DND_DRAG_TYPE_STRING },
 	{ "text/plain",      0, DND_DRAG_TYPE_STRING },
 	{ "text/uri-list",   0, DND_DRAG_TYPE_URL },
 	{ "text/contact-id", 0, DND_DRAG_TYPE_CONTACT_ID },
 };
 
-static GtkTargetEntry drag_types_source[] = {
+static const GtkTargetEntry drag_types_source[] = {
 	{ "text/contact-id", 0, DND_DRAG_TYPE_CONTACT_ID },
 };
 
@@ -510,12 +510,12 @@ gossip_contact_list_init (GossipContactList *list)
 
 	gtk_item_factory_create_items (priv->item_popup_factory,
 				       G_N_ELEMENTS (item_menu_items),
-				       item_menu_items,
+				       (GtkItemFactoryEntry *) item_menu_items,
                                        list);
 	
 	gtk_item_factory_create_items (priv->group_popup_factory,
 				       G_N_ELEMENTS (group_menu_items),
-				       group_menu_items,
+				       (GtkItemFactoryEntry *) group_menu_items,
 				       list);
 	
         /* Signal connection. */
@@ -576,6 +576,16 @@ gossip_contact_list_init (GossipContactList *list)
 static void
 contact_list_finalize (GObject *object)
 {
+	GossipContactListPriv *priv;
+
+	priv = GET_PRIV (object);
+
+	g_hash_table_destroy (priv->groups);
+	g_hash_table_destroy (priv->flash_table);
+
+	gtk_object_destroy (GTK_OBJECT (priv->item_popup_factory));
+	gtk_object_destroy (GTK_OBJECT (priv->group_popup_factory));
+	
 	G_OBJECT_CLASS (gossip_contact_list_parent_class)->finalize (object);
 }
 
@@ -688,8 +698,10 @@ contact_list_contact_updated_cb (GossipSession     *session,
 
 	type = gossip_contact_get_type (contact);
 	if (type != GOSSIP_CONTACT_TYPE_CONTACTLIST) {
-		gossip_debug (DEBUG_DOMAIN, "Update to none-contact list "
-			      "contact (doing nothing)");
+		gossip_debug (DEBUG_DOMAIN,
+			      "Update to none-contact list "
+			      "contact %s (doing nothing)",
+			      gossip_contact_get_name (contact));
 		return;
 	}
 	
@@ -774,6 +786,7 @@ contact_list_contact_presence_updated_cb (GossipSession     *session,
 		if (account) {
 			seconds = gossip_session_get_connected_time (session, 
 								     account);
+			g_object_unref (account);
 		}
 
 		if (priv->show_active) {
@@ -793,6 +806,7 @@ contact_list_contact_presence_updated_cb (GossipSession     *session,
 		if (account) {
 			seconds = gossip_session_get_connected_time (session, 
 								     account);
+			g_object_unref (account);
 		}
 
 		contact_list_add_contact (list, contact);
@@ -802,7 +816,6 @@ contact_list_contact_presence_updated_cb (GossipSession     *session,
 
 			gossip_debug (DEBUG_DOMAIN, "Set active (contact added)");
 		}
-
 	} else {
 		/* Get online state before. */
 		if (iters && g_list_length (iters) > 0) {
@@ -1336,6 +1349,8 @@ contact_list_create_model (GossipContactList *list)
 					      GTK_SORT_ASCENDING);
 
 	gtk_tree_view_set_model (GTK_TREE_VIEW (list), model);
+
+	g_object_unref (model);
 }
 
 static gboolean
@@ -2078,7 +2093,7 @@ contact_list_sort_func (GtkTreeModel *model,
 	} else if (contact_a && !contact_b) {
 		ret_val = -1;
 	} else {
-		ret_val = g_ascii_strcasecmp (name_a, name_b);
+		ret_val = g_utf8_collate (name_a, name_b);
 	}
 
 	g_free (name_a);
@@ -2162,7 +2177,7 @@ contact_list_find_contact_foreach (GtkTreeModel *model,
 static gchar *
 contact_list_item_factory_translate_func (const gchar *path, gpointer data)
 {
-	return _((gchar *) path);
+	return _(path);
 }
 
 static void
@@ -2288,6 +2303,8 @@ contact_list_item_menu_send_file_cb (gpointer   data,
         }
 	
         gossip_ft_window_send_file (contact);
+
+	g_object_unref (contact);
 }
 
 static void
@@ -2726,6 +2743,7 @@ gossip_contact_list_get_selected_group (GossipContactList *list)
 			    -1);
 
 	if (!is_group) {
+		g_free (name);
 		return NULL;
 	}
 
