@@ -22,6 +22,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <dbus/dbus-glib-bindings.h>
+#include <dbus/dbus-glib-lowlevel.h>
 
 #include <libgossip/gossip-debug.h>
 
@@ -35,7 +36,7 @@
 
 /* None-generated functions */
 static DBusGProxy *dbus_freedesktop_init (void);
-static DBusGProxy *dbus_nm_init          (void);
+static gboolean    dbus_nm_init          (void);
 static gboolean    dbus_gossip_show      (gboolean show);
 
 /* Set up the DBus GObject to use */
@@ -94,6 +95,13 @@ gboolean gossip_dbus_toggle_roster      (GossipDBus   *obj,
 #include "gossip-dbus-glue.h"
 
 static GossipSession *saved_session = NULL;
+static GossipDBus    *gossip_dbus = NULL;
+static DBusGProxy    *bus_proxy = NULL;
+
+static DBusGProxy    *nm_proxy = NULL;
+static gint           nm_proxy_restart_retries = 0;
+static guint          nm_proxy_restart_timeout_id = 0;
+
 
 static void
 gossip_dbus_init (GossipDBus *obj)
@@ -275,9 +283,8 @@ gossip_dbus_init_for_session (GossipSession *session,
 			      gboolean       multiple_instances)
 {
 	DBusGConnection *bus;
-	DBusGProxy      *bus_proxy;
+	DBusGProxy      *proxy;
 	GError          *error = NULL;
-	GossipDBus      *obj;
 	guint            result;
 
 	g_return_val_if_fail (GOSSIP_IS_SESSION (session), FALSE);
@@ -296,18 +303,19 @@ gossip_dbus_init_for_session (GossipSession *session,
 		g_warning ("Couldn't connect to the session bus");
 		return FALSE;
 	}
-
-	if ((bus_proxy = dbus_freedesktop_init ()) == NULL) {
+	
+	proxy = dbus_freedesktop_init ();
+	if (!proxy) {
 		return FALSE;
 	}
 
 	dbus_g_object_type_install_info (GOSSIP_TYPE_DBUS, &dbus_glib_gossip_dbus_object_info);
 
-	obj = g_object_new (GOSSIP_TYPE_DBUS, NULL);
+	gossip_dbus = g_object_new (GOSSIP_TYPE_DBUS, NULL);
 
-	dbus_g_connection_register_g_object (bus, GOSSIP_DBUS_PATH, G_OBJECT (obj));
+	dbus_g_connection_register_g_object (bus, GOSSIP_DBUS_PATH, G_OBJECT (gossip_dbus));
 	
-	if (!org_freedesktop_DBus_request_name (bus_proxy,
+	if (!org_freedesktop_DBus_request_name (proxy,
 						GOSSIP_DBUS_SERVICE,
 						DBUS_NAME_FLAG_DO_NOT_QUEUE,
 						&result, &error)) {
@@ -320,12 +328,14 @@ gossip_dbus_init_for_session (GossipSession *session,
 
 	if (!multiple_instances && result == DBUS_REQUEST_NAME_REPLY_EXISTS) {
 		gossip_debug (DEBUG_DOMAIN, "Gossip is already running");
-		
+
 		dbus_gossip_show (TRUE);
+		gdk_notify_startup_complete ();
+
 		exit (EXIT_SUCCESS);
 	}
 	
-	/* Set up GNOME Netwrk Manager if available so we get signals */
+	/* Set up GNOME Network Manager if available so we get signals. */
 	dbus_nm_init ();
 
 	gossip_debug (DEBUG_DOMAIN, "Ready");
@@ -333,10 +343,13 @@ gossip_dbus_init_for_session (GossipSession *session,
 	return TRUE;
 }
 
-
 void 
 gossip_dbus_finalize_for_session (void)
 {
+	g_object_unref (bus_proxy);
+	g_object_unref (nm_proxy);
+
+	g_object_unref (gossip_dbus);
 	g_object_unref (saved_session);
 }
 
@@ -351,9 +364,8 @@ gossip_dbus_finalize_for_session (void)
 static DBusGProxy *
 dbus_freedesktop_init (void)
 {
-	DBusGConnection   *bus;
-	static DBusGProxy *bus_proxy = NULL;
-	GError            *error = NULL;
+	DBusGConnection *bus;
+	GError          *error = NULL;
 
 	if (bus_proxy) {
 		return bus_proxy;
@@ -366,12 +378,12 @@ dbus_freedesktop_init (void)
 		g_warning ("Couldn't connect to the session bus");
 		return NULL;
 	}
-	
-	bus_proxy = dbus_g_proxy_new_for_name (bus, 
+
+	bus_proxy = dbus_g_proxy_new_for_name (bus,
 					       FREEDESKTOP_DBUS_SERVICE,
 					       FREEDESKTOP_DBUS_PATH,
 					       FREEDESKTOP_DBUS_INTERFACE);
-	
+
 	if (!bus_proxy) {
 		g_warning ("Could not connect to Freedesktop");
 	}
@@ -435,82 +447,110 @@ dbus_nm_state_cb (DBusGProxy *proxy,
 	}
 }
 
-static DBusGProxy *
+static gboolean
+nm_proxy_restart_timeout_cb (gpointer user_data)
+{
+	if (dbus_nm_init ()) {
+		nm_proxy_restart_timeout_id = 0;
+		return FALSE;
+	}
+
+	nm_proxy_restart_retries--;
+	if (nm_proxy_restart_retries == 0) {
+		nm_proxy_restart_timeout_id = 0;
+		return FALSE;
+	}
+
+	return TRUE;
+}
+		
+static void
+nm_proxy_notify_cb (gpointer conn,
+		    GObject *where_the_object_was)
+{
+	dbus_connection_unref (conn);
+
+	nm_proxy = NULL;
+	nm_proxy_restart_retries = 5;
+
+	if (nm_proxy_restart_timeout_id) {
+		g_source_remove (nm_proxy_restart_timeout_id);
+	}
+	
+	nm_proxy_restart_timeout_id = 
+		g_timeout_add (10*1000,
+			       nm_proxy_restart_timeout_cb,
+			       NULL);
+}
+
+static gboolean
 dbus_nm_init (void)
 {
-	DBusGConnection   *bus;
-	static DBusGProxy *bus_proxy = NULL;
-	GError            *error = NULL;
+	DBusGConnection *bus;
+	DBusConnection  *conn;
 
-	if (bus_proxy) {
-		return bus_proxy;
+	if (nm_proxy) {
+		return TRUE;
 	}
 
 	gossip_debug (DEBUG_DOMAIN, "Initialising Network Manager proxy");
 
-	bus = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);
+	bus = dbus_g_bus_get (DBUS_BUS_SYSTEM, NULL);
 	if (!bus) {
 		g_warning ("Could not connect to system bus");
 		return FALSE;
 	}
+
+	conn = dbus_g_connection_get_connection (bus);
+	dbus_connection_set_exit_on_disconnect (conn, FALSE);
 	
-	bus_proxy = dbus_g_proxy_new_for_name (bus, 
+	nm_proxy = dbus_g_proxy_new_for_name (bus,
 					       NM_DBUS_SERVICE, 
 					       NM_DBUS_PATH, 
 					       NM_DBUS_INTERFACE);
-	
-	if (!bus_proxy) {
+
+	if (!nm_proxy) {
 		g_warning ("Could not connect to Network Manager");
-		return NULL;
+		return FALSE;
 	}
 
+	g_object_weak_ref (G_OBJECT (nm_proxy), nm_proxy_notify_cb, conn);
+	
 	dbus_g_object_register_marshaller (g_cclosure_marshal_VOID__UINT, 
 					   G_TYPE_NONE, G_TYPE_UINT, G_TYPE_INVALID);
 
-	/* Tell DBus what the type signature of the signal callback is; this
-	 * allows us to sanity-check incoming messages before invoking the
-	 * callback.  You need to do this once for each proxy you create,
-	 * not every time you want to connect to the signal.
-	 */
-	dbus_g_proxy_add_signal (bus_proxy, "StateChange",
+	dbus_g_proxy_add_signal (nm_proxy, "StateChange",
 				 G_TYPE_UINT, G_TYPE_INVALID);
 
-	/* Actually connect to the signal. Note you can call
-	 * dbus_g_proxy_connect_signal multiple times for one invocation of
-	 * dbus_g_proxy_add_signal.
-	 */
-	dbus_g_proxy_connect_signal (bus_proxy, "StateChange", 
+	dbus_g_proxy_connect_signal (nm_proxy, "StateChange", 
 				     G_CALLBACK (dbus_nm_state_cb),
 				     NULL, NULL);
 
-	return bus_proxy;
+	return TRUE;
 } 
 
 gboolean 
 gossip_dbus_nm_get_state (gboolean *connected)
 {
-	DBusGConnection *bus;
-	DBusGProxy      *bus_proxy;
-	GError          *error = NULL;
-	guint32          state;
+	GError  *error = NULL;
+	guint32  state;
 	
 	g_return_val_if_fail (connected != NULL, FALSE);
 
 	/* Set the initial value of connected incase we have to return */
 	*connected = FALSE;
 
-	/* Make sure we have set up Network Manager connections */
-	if ((bus_proxy = dbus_nm_init ()) == NULL) {
+	if (nm_proxy_restart_timeout_id) {
+		/* We are still trying to reconnect to the restarted bus. */
 		return FALSE;
 	}
 
-	bus = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);
-	if (!bus) {
-		g_warning ("Could not connect to system bus");
+	/* Make sure we have set up Network Manager connections */
+	if (!dbus_nm_init ()) {
 		return FALSE;
 	}
-	
-	if (!dbus_g_proxy_call (bus_proxy, "state", &error,
+
+	if (!dbus_g_proxy_call (nm_proxy, "state", &error,
 				G_TYPE_INVALID, 
 				G_TYPE_UINT, &state, G_TYPE_INVALID)) {
 		gossip_debug (DEBUG_DOMAIN, "Failed to complete 'state' request. %s", 
@@ -535,30 +575,30 @@ static gboolean
 dbus_gossip_show (gboolean show)
 {
 	DBusGConnection *bus;
-	DBusGProxy      *bus_proxy;
+	DBusGProxy      *proxy;
 	GError          *error = NULL;
 	gboolean         success = TRUE;
 	
 	bus = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
 	if (!bus) {
-		g_warning ("Could not connect to system bus");
+		g_warning ("Could not connect to session bus");
 		return FALSE;
 	}
 
 	/* We are doing this because normally we wouldn't have a
 	 * connection to ourselves.
 	 */
-	bus_proxy = dbus_g_proxy_new_for_name (bus, 
-					       GOSSIP_DBUS_SERVICE,
-					       GOSSIP_DBUS_PATH,
-					       GOSSIP_DBUS_INTERFACE);
-
-	if (!bus_proxy) {
+	proxy = dbus_g_proxy_new_for_name (bus, 
+					   GOSSIP_DBUS_SERVICE,
+					   GOSSIP_DBUS_PATH,
+					   GOSSIP_DBUS_INTERFACE);
+	
+	if (!proxy) {
 		g_warning ("Could not connect to other instance of Gossip");
 		return FALSE;
 	}
 		
-	if (!dbus_g_proxy_call (bus_proxy, "SetRosterVisible", &error,
+	if (!dbus_g_proxy_call (proxy, "SetRosterVisible", &error,
 				G_TYPE_BOOLEAN, show,
 				G_TYPE_INVALID,
 				G_TYPE_INVALID)) {
@@ -567,7 +607,7 @@ dbus_gossip_show (gboolean show)
 		success = FALSE;
 	}
 
-	g_object_unref (bus_proxy);
+	g_object_unref (proxy);
 
 	return success;
 }
