@@ -53,22 +53,14 @@
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
 
-#ifdef USE_GNOMEVFS_FOR_URL
-#include <libgnomevfs/gnome-vfs.h>
-#else
-#include <libgnome/gnome-url.h>
-#endif
-
-#include <libgnomevfs/gnome-vfs-utils.h>
-#include <libxslt/xslt.h>
-#include <libxslt/transform.h>
-#include <libxslt/xsltutils.h>
-#include "libexslt/exslt.h"
-
 #include <libgossip/gossip-debug.h>
 #include <libgossip/gossip-message.h>
 #include <libgossip/gossip-utils.h>
 
+#ifdef HAVE_GNOME
+#include <libgnomevfs/gnome-vfs-utils.h>
+#endif
+
 #include "gossip-app.h"
 #include "gossip-log.h"
 
@@ -115,6 +107,8 @@ struct _GossipLogSearchHit {
 	gchar         *date;
 };
 
+static gchar *         log_escape                              (const gchar    *str);
+static gchar *         log_unescape                            (const gchar    *str);
 static void            log_handlers_notify_foreach             (GossipLogMessageFunc   func,
 								HandlerData           *data,
 								GList                **list);
@@ -150,15 +144,30 @@ static gboolean        log_set_name                            (GossipAccount
 static gboolean        log_set_own_name                        (GossipAccount   *account,
 								GossipContact   *contact);
 
-#ifdef DEPRECATED 
-static gboolean        log_transform                           (const gchar     *infile,
-							        gint             fd);
-static void            log_show                                (GossipLog       *log);
-#endif
 
 static GHashTable *contact_files = NULL;
 static GHashTable *message_handlers = NULL;
 
+static gchar *
+log_escape (const gchar *str)
+{
+#ifdef HAVE_GNOME
+	return gnome_vfs_escape_host_and_path_string (str);
+#else
+	return g_strdup (str);
+#endif
+}
+
+static gchar *
+log_unescape (const gchar *str)
+{
+#ifdef HAVE_GNOME
+	return gnome_vfs_unescape_string_for_display (str);
+#else
+	return g_strdup (str);
+#endif
+}
+
 static void
 log_handler_free (HandlerData *data)
 {
@@ -322,7 +331,9 @@ log_get_timestamp_from_message (GossipMessage *message)
 	gossip_time_t t;
 
 	t = gossip_message_get_timestamp (message);
-	return gossip_time_to_timestamp_full (t, LOG_TIME_FORMAT_FULL);
+
+	/* We keep the timestamps in the messages as UTC. */
+	return gossip_time_to_string_utc (t, LOG_TIME_FORMAT_FULL);
 }
 
 static gchar *
@@ -331,7 +342,7 @@ log_get_timestamp_filename (void)
 	gossip_time_t t;
 
 	t = gossip_time_get_current ();
-	return gossip_time_to_timestamp_full (t, LOG_TIME_FORMAT);
+	return gossip_time_to_string_local (t, LOG_TIME_FORMAT);
 }
 
 static gchar *
@@ -362,6 +373,7 @@ log_get_account_id_from_filename (const gchar *filename)
 	gchar       *log_directory;
 	const gchar *p1;
 	const gchar *p2;
+	gchar       *ret;
 
 	log_check_dir (&log_directory);
 	
@@ -379,7 +391,9 @@ log_get_account_id_from_filename (const gchar *filename)
 		return NULL;
 	}
 
-	return g_strndup (p1, p2 - p1); 
+	ret = g_strndup (p1, p2 - p1);
+	g_free (log_directory);
+	return ret;
 }
 
 static GossipChatroom *
@@ -587,6 +601,8 @@ log_get_all_log_files (GList **files)
 	*files = NULL;
 
 	if (log_check_dir (&log_directory)) {
+		g_free (log_directory);
+
 		gossip_debug (DEBUG_DOMAIN, "No log directory exists");
 		return TRUE;
 	}
@@ -637,12 +653,12 @@ log_get_filename_by_date_for_contact (GossipContact *contact,
 	}
 	
 	account_id = gossip_account_get_id (account);
-	account_id_escaped = gnome_vfs_escape_host_and_path_string (account_id);
+	account_id_escaped = log_escape (account_id);
 
 	log_check_dir (&log_directory);
 
 	contact_id = gossip_contact_get_id (contact);
-	contact_id_escaped = gnome_vfs_escape_host_and_path_string (contact_id);
+	contact_id_escaped = log_escape (contact_id);
 	
 	basename = g_strconcat (date, LOG_FILENAME_SUFFIX, NULL);
 	filename = g_build_filename (log_directory,
@@ -703,12 +719,12 @@ log_get_filename_by_date_for_chatroom (GossipChatroom *chatroom,
 	}
 	
 	account_id = gossip_account_get_id (account);
-	account_id_escaped = gnome_vfs_escape_host_and_path_string (account_id);
+	account_id_escaped = log_escape (account_id);
 
 	log_check_dir (&log_directory);
 
 	chatroom_id = gossip_chatroom_get_id_str (chatroom);
-	chatroom_id_escaped = gnome_vfs_escape_host_and_path_string (chatroom_id);
+	chatroom_id_escaped = log_escape (chatroom_id);
 	
 	basename = g_strconcat (date, LOG_FILENAME_SUFFIX, NULL);
 	filename = g_build_filename (log_directory,
@@ -765,7 +781,7 @@ log_get_contacts_filename (GossipAccount *account)
 		    gossip_account_get_id (account)); 
 
 	account_id = gossip_account_get_id (account);
-	account_id_escaped = gnome_vfs_escape_host_and_path_string (account_id);
+	account_id_escaped = log_escape (account_id);
 
 	log_check_dir (&log_directory);
 
@@ -951,128 +967,6 @@ log_set_own_name (GossipAccount *account,
 	return ok;
 }
 
-#ifdef DEPRECATED
-
-static gboolean
-log_transform (const gchar *infile, 
-	       gint         fd)
-{
-        xsltStylesheet  *stylesheet;
-        xmlDoc          *xml_doc;
-        xmlDoc          *html_doc;
-	gchar           *title;
-	const gchar     *params[6];
-
-	/* Setup libxml. */
-	xmlSubstituteEntitiesDefault (1);
-	xmlLoadExtDtdDefaultValue = 1;
-	exsltRegisterAll ();
-
-        stylesheet = xsltParseStylesheetFile ((const xmlChar *)STYLESHEETDIR "/gossip-log.xsl");
-	if (!stylesheet) {
-		return FALSE;
-	}
-
-        xml_doc = xmlParseFile (infile);
-	if (!xml_doc) {
-		return FALSE;
-	}
-
-	title = g_strconcat ("\"", _("Conversation Log"), "\"", NULL);
-
-	/* Set params to be passed to stylesheet. */
-	params[0] = "title";
-	params[1] = title;
-        params[2] = NULL;
-
-        html_doc = xsltApplyStylesheet (stylesheet, xml_doc, params);
-	if (!html_doc) {
-		xmlFreeDoc (xml_doc);
-		return FALSE;
-	}
-
-        xmlFreeDoc (xml_doc);
-
-	xsltSaveResultToFd (fd, html_doc, stylesheet);
-
-	xsltFreeStylesheet (stylesheet);
-        xmlFreeDoc (html_doc);
-
-	g_free (title);
-
-	return TRUE;
-}
-
-static void
-log_show (GossipLog *log)
-{
-	GossipLogPriv  *priv;
-#ifdef USE_GNOMEVFS_FOR_URL
-	GnomeVFSResult  result;
-#endif
-	gchar          *filename = NULL;
-	gchar          *outfile;
-	gchar          *url;
-	FILE           *file;
-	gint            fd;
-
-	priv = GET_PRIV (log);
-
-	if (!g_file_test (priv->filename, G_FILE_TEST_EXISTS)) {
-		file = g_fopen (priv->filename, "w+");
-		if (file) {
-			g_fprintf (file, LOG_HEADER LOG_FOOTER);
-			fclose (file);
-		}
-	}
-
-	if (!g_file_test (priv->filename, G_FILE_TEST_EXISTS)) {
-		return;
-	}
-
-	fd = g_file_open_tmp ("gossip-log-XXXXXX", &outfile, NULL);
-	if (fd == -1) {
-		return;
-	}
-
-	gossip_debug (DEBUG_DOMAIN, "Using temporary file:'%s'", outfile);
-
-	if (!log_transform (priv->filename, fd)) {
-		g_warning ("Couldn't transform log file:'%s'", outfile);
-		g_free (outfile);
-		return;
-	}
-
-	close (fd);
-
-	filename = g_strconcat (outfile, ".html", NULL);
-	if (rename (outfile, filename) == -1) {
-		g_warning ("Couldn't rename temporary log file:'%s' to '%s'", 
-			   outfile, filename);
-		g_free (outfile);
-		g_free (filename);
-		return;
-	}
-
-	g_free (outfile);
-
-	url = g_strconcat (LOG_FILENAME_PREFIX, filename, NULL);
-
-#ifdef USE_GNOMEVFS_FOR_URL
-	result = gnome_vfs_url_show (url);
-	if (result == GNOME_VFS_OK) {
-		g_warning ("Couldn't show log file:'%s'", filename);
-	}
-#else
-	gnome_url_show (url, NULL);
-#endif
-
-	g_free (url);
-	g_free (filename);
-}
-
-#endif /* DEPRECATED */
-
 GList *
 gossip_log_get_contacts (GossipAccount *account)
 {
@@ -1094,7 +988,7 @@ gossip_log_get_contacts (GossipAccount *account)
 	}
 
 	account_id = gossip_account_get_id (account);
-	account_id_escaped = gnome_vfs_escape_host_and_path_string (account_id);
+	account_id_escaped = log_escape (account_id);
 
 	directory = g_build_path (G_DIR_SEPARATOR_S, 
 				  log_directory, 
@@ -1122,7 +1016,7 @@ gossip_log_get_contacts (GossipAccount *account)
 			continue;
 		}
 
- 		filename_unescaped = gnome_vfs_unescape_string_for_display (filename);
+ 		filename_unescaped = log_unescape (filename);
 		contact = log_get_contact (account, filename_unescaped);
 		g_free (filename_unescaped);
 
@@ -1155,12 +1049,13 @@ gossip_log_get_chatrooms (GossipAccount *account)
 	g_return_val_if_fail (GOSSIP_IS_ACCOUNT (account), NULL);
 
 	if (log_check_dir (&log_directory)) {
+		g_free (log_directory);
 		gossip_debug (DEBUG_DOMAIN, "No log directory exists");
 		return NULL;
 	}
 
 	account_id = gossip_account_get_id (account);
-	account_id_escaped = gnome_vfs_escape_host_and_path_string (account_id);
+	account_id_escaped = log_escape (account_id);
 
 	directory = g_build_path (G_DIR_SEPARATOR_S, 
 				  log_directory, 
@@ -1191,7 +1086,7 @@ gossip_log_get_chatrooms (GossipAccount *account)
 
 		g_free (full);
 		
- 		filename_unescaped = gnome_vfs_unescape_string_for_display (filename);
+ 		filename_unescaped = log_unescape (filename);
 		chatroom = log_get_chatroom_from_filename (account, filename_unescaped);
 
 		g_free (filename_unescaped);
@@ -1209,13 +1104,16 @@ gossip_log_get_chatrooms (GossipAccount *account)
 	return chatrooms;
 }
 
+// FIXME: Move to gossip-log-window.c
 gchar *
 gossip_log_get_date_readable (const gchar *date)
 {
 	gossip_time_t t;
-	
-	t = gossip_time_from_string_full (date, LOG_TIME_FORMAT);
-	return gossip_time_to_timestamp_full (t, "%a %d %b %Y");
+
+	// koko this won't work as the input doesn't have the time.
+	t = gossip_time_parse (date); //, LOG_TIME_FORMAT);
+
+	return gossip_time_to_string_local (t, "%a %d %b %Y");
 }
 
 /*
@@ -1240,15 +1138,16 @@ gossip_log_get_dates_for_contact (GossipContact *contact)
 	account = gossip_contact_get_account (contact);
 	
 	if (log_check_dir (&log_directory)) {
+		g_free (log_directory);
 		gossip_debug (DEBUG_DOMAIN, "No log directory exists");
 		return NULL;
 	}
 
 	account_id = gossip_account_get_id (account);
-	account_id_escaped = gnome_vfs_escape_host_and_path_string (account_id);
+	account_id_escaped = log_escape (account_id);
 
 	contact_id = gossip_contact_get_id (contact);
-	contact_id_escaped = gnome_vfs_escape_host_and_path_string (contact_id);
+	contact_id_escaped = log_escape (contact_id);
 
 	directory = g_build_path (G_DIR_SEPARATOR_S, 
 				  log_directory, 
@@ -1405,7 +1304,7 @@ gossip_log_get_messages_for_contact (GossipContact *contact,
 		body = xmlNodeGetContent (node);
 		
 		time = xmlGetProp (node, "time");
-		t = gossip_time_from_string_full (time, "%Y%m%dT%H:%M:%S");
+		t = gossip_time_parse (time);
 		
 		name = xmlGetProp (node, "nick");
 		
@@ -1648,32 +1547,6 @@ gossip_log_get_last_for_contact (GossipContact *contact)
 	return messages;
 }
 
-#ifdef DEPRECATED
-void
-gossip_log_show_for_contact (GtkWidget     *window, 
-			     GossipContact *contact)
-{
-	GdkCursor *cursor;
-	GossipLog *log;
-
-	cursor = gdk_cursor_new (GDK_WATCH);
-
-	gdk_window_set_cursor (window->window, cursor);
-	gdk_cursor_unref (cursor);
-
-	/* Let the UI redraw before we start the slow transformation. */
-	while (gtk_events_pending ()) {
-		gtk_main_iteration ();
-	}
-
-	log = gossip_log_lookup_for_contact (contact);
-	
-	log_show (log);
-
-	gdk_window_set_cursor (window->window, NULL);
-}
-#endif /* DEPRECATED */
-
 /*
  * Chatroom functions 
  */
@@ -1699,15 +1572,16 @@ gossip_log_get_dates_for_chatroom (GossipChatroom *chatroom)
 	}
 	
 	if (log_check_dir (&log_directory)) {
+		g_free (log_directory);
 		gossip_debug (DEBUG_DOMAIN, "No log directory exists");
 		return NULL;
 	}
 
 	account_id = gossip_account_get_id (account);
-	account_id_escaped = gnome_vfs_escape_host_and_path_string (account_id);
+	account_id_escaped = log_escape (account_id);
 
 	chatroom_id = gossip_chatroom_get_id_str (chatroom);
-	chatroom_id_escaped = gnome_vfs_escape_host_and_path_string (chatroom_id);
+	chatroom_id_escaped = log_escape (chatroom_id);
 	
 	directory = g_build_path (G_DIR_SEPARATOR_S, 
 				  log_directory, 
@@ -1835,7 +1709,7 @@ gossip_log_get_messages_for_chatroom (GossipChatroom *chatroom,
 		body = xmlNodeGetContent (node);
 		
 		time = xmlGetProp (node, "time");
-		t = gossip_time_from_string_full (time, "%Y%m%dT%H:%M:%S");
+		t = gossip_time_parse (time);
 
 		sent = gossip_contact_equal (contact, own_contact);
 		
@@ -1994,16 +1868,14 @@ gossip_log_exists_for_chatroom (GossipChatroom *chatroom)
 	return exists;
 }
 
-#ifdef DEPRECATED 
-void
-gossip_log_show_for_chatroom (GtkWidget      *window,
-			      GossipChatroom *chatroom)
+/* FIXME: Use this code for searching since that is really slow. */
+#if 0
+static void
+log_foo (GtkWidget *window)
 {
 	GdkCursor *cursor;
-	GossipLog *log;
 
 	cursor = gdk_cursor_new (GDK_WATCH);
-
 	gdk_window_set_cursor (window->window, cursor);
 	gdk_cursor_unref (cursor);
 
@@ -2012,13 +1884,11 @@ gossip_log_show_for_chatroom (GtkWidget      *window,
 		gtk_main_iteration ();
 	}
 
-	log = gossip_log_lookup_for_chatroom (chatroom);
-	
-	log_show (log);
+	// ...
 
 	gdk_window_set_cursor (window->window, NULL);	
 }
-#endif /* DEPRECATED */
+#endif
 
 /*
  * Searching
@@ -2048,26 +1918,30 @@ gossip_log_search_new (const gchar *text)
 	}
 
 	session = gossip_app_get_session ();
- 	manager = gossip_session_get_account_manager (session);
+	manager = gossip_session_get_account_manager (session);
 
 	for (l = files; l; l = l->next) {
+		GMappedFile *file;
+		gsize        length;
+
 		filename = l->data;
 
-		if (!g_file_get_contents (filename, &contents, NULL, NULL)) {
-			continue;
-		}
-		
-		if (!contents || strlen (contents) < 1) {
-			continue;
-		}
-		
 		/* FIXME: Handle chatrooms */
 		if (strstr (filename, LOG_DIR_CHATROOMS)) {
 			continue;
 		}
 
-		contents_casefold = g_utf8_casefold (contents, -1);
-		g_free (contents);
+		file = g_mapped_file_new (filename, FALSE, NULL);
+		if (!file) {
+			continue;
+		}
+
+		length = g_mapped_file_get_length (file);
+		contents = g_mapped_file_get_contents (file);
+
+		contents_casefold = g_utf8_casefold (contents, length);
+
+		g_mapped_file_free (file);
 
 		if (strstr (contents_casefold, text_casefold)) {
 			GossipLogSearchHit *hit;
