@@ -21,15 +21,19 @@
 #include <config.h>
 #include <string.h>
 #include <stdlib.h>
-
-#include <gtk/gtkwidget.h>
-#include <gtk/gtkwindow.h>
-#include <gtk/gtkmain.h>
+#include <gtk/gtk.h>
+#include <glib.h>
 #include <glib/gi18n.h>
-#include <glib/goption.h>
 
+#ifdef HAVE_GNOME
 #include <libgnome/gnome-program.h>
 #include <libgnomeui/gnome-ui-init.h>
+#endif
+
+#include <libgossip/gossip-account-manager.h>
+#include <libgossip/gossip-account.h>
+#include <libgossip/gossip-session.h>
+#include <libgossip/gossip-paths.h>
 
 #ifdef HAVE_DBUS
 #include "gossip-dbus.h"
@@ -39,71 +43,80 @@
 #include "gossip-galago.h"
 #endif
 
-#include <libgossip/gossip-account-manager.h>
-#include <libgossip/gossip-account.h>
-#include <libgossip/gossip-session.h>
-
 #include "gossip-preferences.h"
 #include "gossip-stock.h"
-#include "gossip-dbus.h"
 #include "gossip-app.h"
 
-#define GNOME_PARAM_GOPTION_CONTEXT "goption-context"
-
 static gboolean  no_connect = FALSE;
 static gboolean  multiple_instances = FALSE;
 static gboolean  list_accounts = FALSE;
 static gchar    *account_name = NULL;
 
-static const GOptionEntry options[] = {
-	{ "no-connect", 'n', 
-	  0, G_OPTION_ARG_NONE, &no_connect,
-	  N_("Don't connect on startup"),
-	  NULL },
-	{ "multiple-instances", 'm', 
-	  0, G_OPTION_ARG_NONE, &multiple_instances,
-	  N_("Allow multiple instances of the application to run at the same time"),
-	  NULL },
-	{ "list-accounts", 'l', 
-	  0, G_OPTION_ARG_NONE, &list_accounts,
-	  N_("List the available accounts"),
-	  NULL },
-	{ "account", 'a', 
-	  0, G_OPTION_ARG_STRING, &account_name,
-	  N_("Which account to connect to on startup"),
-	  N_("ACCOUNT-NAME") },
-	{ NULL }
-};	
-
 int
 main (int argc, char *argv[])
 {
+	gchar                *localedir;
+#ifdef HAVE_GNOME
 	GnomeProgram         *program;
+#else
+	GError               *error = NULL;
+#endif
 	GossipSession        *session;
 	GossipAccountManager *account_manager;
 	GossipAccount        *account = NULL;
 	GOptionContext       *context;
 	GList                *accounts;
-	
-	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+	GOptionEntry          options[] = {
+		{ "no-connect", 'n', 
+		  0, G_OPTION_ARG_NONE, &no_connect,
+		  N_("Don't connect on startup"),
+		  NULL },
+		{ "multiple-instances", 'm', 
+		  0, G_OPTION_ARG_NONE, &multiple_instances,
+		  N_("Allow multiple instances of the application to run at the same time"),
+		  NULL },
+		{ "list-accounts", 'l', 
+		  0, G_OPTION_ARG_NONE, &list_accounts,
+		  N_("List the available accounts"),
+		  NULL },
+		{ "account", 'a', 
+		  0, G_OPTION_ARG_STRING, &account_name,
+		  N_("Which account to connect to on startup"),
+		  N_("ACCOUNT-NAME") },
+		{ NULL }
+	};
+
+	localedir = gossip_paths_get_locale_path ();
+	bindtextdomain (GETTEXT_PACKAGE, localedir);
         bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
+	g_free (localedir);
 
 	context = g_option_context_new (_("- Gossip Instant Messenger"));
 	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);	
 
+	g_set_application_name (PACKAGE_NAME);
+
+#ifdef HAVE_GNOME
 	program = gnome_program_init ("gossip", PACKAGE_VERSION,
 				      LIBGNOMEUI_MODULE,
                                       argc, argv,
                                       GNOME_PROGRAM_STANDARD_PROPERTIES,
-				      GNOME_PARAM_GOPTION_CONTEXT, context,
+				      "goption-context", context,
 				      GNOME_PARAM_HUMAN_READABLE_NAME, PACKAGE_NAME,
 				      NULL);
+#else
+	if (!gtk_init_with_args (&argc, &argv,
+				 _("- Gossip Instant Messenger"),
+				 options,
+				 GETTEXT_PACKAGE,
+				 &error)) {
+		g_printerr ("%s\n", error->message);
+		return 1;
+	}
+#endif
 
-	g_option_context_free (context);
-
-	g_set_application_name (PACKAGE_NAME);
-	gtk_window_set_default_icon_name ("gossip");
+	gossip_window_set_default_icon_name ("gossip");
 
 	/* Get all accounts. */
  	account_manager = gossip_account_manager_new (NULL);
@@ -134,7 +147,7 @@ main (int argc, char *argv[])
 			GossipAccount *account = l->data;
 			
 			g_print (" %s", gossip_account_get_name (account));
-			if (gossip_account_equal (account, def)) {
+			if (def && gossip_account_equal (account, def)) {
 				g_printerr (" ");
 				g_printerr (_("[default]"));
 			}
@@ -174,6 +187,7 @@ main (int argc, char *argv[])
 
 	gossip_app_create (session, account_manager);
 	g_object_unref (session);
+	g_object_unref (account_manager);
 	
 	if (!no_connect) {
 		gossip_app_connect (account, TRUE);
@@ -181,10 +195,35 @@ main (int argc, char *argv[])
 	
 	gtk_main ();
 
-	g_object_unref (session);
-	g_object_unref (account_manager);
+#ifdef HAVE_DBUS
+	gossip_dbus_finalize_for_session ();
+#endif
 
+	g_object_unref (gossip_app_get ());
+
+	gossip_stock_finalize ();
+
+#ifdef HAVE_GNOME
 	g_object_unref (program);
-
+#endif
+	
 	return EXIT_SUCCESS;
 }
+
+/*
+
+  Still leaked from just starting up, connecting one account, disconnecting it
+  from the menu, quitting:
+
+  1 GossipAccount, 64 bytes
+  2 GossipPresence, 64 bytes
+  1 GossipJabber, 80 bytes
+  2 GossipContact, 104 bytes
+  
+  Having a chat window open and connecting/disconnecting a few times before
+  quitting:
+  
+  5 GossipAccount, 340 bytes
+  10 GossipContact, 520 bytes
+  
+*/
