@@ -24,9 +24,10 @@
 #include <libgossip/gossip-session.h>
 #include <libgossip/gossip-event-manager.h>
 
-void gossip_notify_init            (GossipSession      *session,
-				    GossipEventManager *event_manager);
-void gossip_notify_contact_online  (GossipContact      *contact);
-void gossip_notify_contact_offline (GossipContact      *contact);
+void gossip_notify_init              (GossipSession      *session,
+				      GossipEventManager *event_manager);
+void gossip_notify_finalize          (void);
+
+void gossip_notify_set_attach_widget (GtkWidget          *new_attach_widget);
 
 #endif /* __GOSSIP_NOTIFY_H__ */
