@@ -19,7 +19,6 @@
  */
 
 #include <config.h>
-
 #include <string.h>
 #include <stdlib.h>
 
@@ -234,7 +233,7 @@ presence_chooser_set_state (GossipPresenceChooser *chooser,
 	} else {
 		/* Only store the value if it differs from the default ones. */
 		if (save && strcmp (status, default_status) != 0) {
-			gossip_status_presets_set_last (status, state);
+			gossip_status_presets_set_last (state, status);
 		}
 	}
 
@@ -300,7 +299,7 @@ presence_chooser_show_dialog (GossipPresenceChooser *chooser,
 		dialog = NULL;
 	}
 
-	glade = gossip_glade_get_file (GLADEDIR "/main.glade",
+	glade = gossip_glade_get_file ("main.glade",
 				       "status_message_dialog",
 				       NULL,
 				       "status_message_dialog", &dialog,
