@@ -104,5 +104,15 @@ gboolean   gossip_text_iter_forward_search          (const GtkTextIter   *iter,
 /* Windows */
 gboolean   gossip_window_get_is_visible             (GtkWindow           *window);
 void       gossip_window_present                    (GtkWindow           *window);
+void       gossip_window_set_default_icon_name      (const gchar         *name);
+
+void       gossip_url_show                          (const char          *url);
+void       gossip_help_show                         (void);
+
+GtkWidget *gossip_link_button_new                   (const gchar         *url,
+						     const gchar         *title);
+
+gboolean   gossip_have_tray                         (void);
+void       gossip_request_user_attention            (void);
 
 #endif /*  __GOSSIP_UI_UTILS_H__ */
